#include <shogun/base/init.h>
#include <shogun/base/some.h>
#include <shogun/lib/SGVector.h>
#include <shogun/io/SerializableAsciiFile.h>
#include <shogun/mathematics/Math.h>
#include <shogun/features/DenseFeatures.h>
#include <shogun/io/CSVFile.h>
#include <shogun/lib/WrappedObjectArray.h>
#include <shogun/clustering/GMM.h>
#include <shogun/lib/SGMatrix.h>

using namespace shogun;

int main(int, char*[])
{
init_shogun_with_defaults();

auto f_feats_train = some<CCSVFile>("../../data/classifier_4class_2d_linear_features_train.dat");

CMath::init_random(1);

//![create_features]
auto features_train = some<CDenseFeatures<float64_t>>(f_feats_train);
//![create_features]

//![create_gmm_instance]
auto num_components = 3;
auto gmm = some<CGMM>(num_components);
//![create_gmm_instance]

//![train_sample]
gmm->set_features(features_train);
gmm->train_em();
auto output = gmm->sample();
//![train_sample]

//![training_smem]
gmm->train_smem();
//![training_smem]

//![extract_params]
auto component_num = 1;
auto nth_mean = gmm->get_nth_mean(component_num);
auto nth_cov = gmm->get_nth_cov(component_num);
auto coef = gmm->get_coef();
//![extract_params]

//![cluster_output]
auto log_likelihoods = gmm->cluster(nth_mean);
//![cluster_output]

// Serialize output for integration testing (automatically generated)
auto __sg_storage = some<CWrappedObjectArray>();
auto __sg_storage_file = some<CSerializableAsciiFile>("gmm.dat", 'w');
__sg_storage->append_wrapped_int(num_components, "num_components");
__sg_storage->append_wrapped_real_vector(output, "output");
__sg_storage->append_wrapped_int(component_num, "component_num");
__sg_storage->append_wrapped_real_vector(nth_mean, "nth_mean");
__sg_storage->append_wrapped_real_matrix(nth_cov, "nth_cov");
__sg_storage->append_wrapped_real_vector(coef, "coef");
__sg_storage->append_wrapped_real_vector(log_likelihoods, "log_likelihoods");
__sg_storage->save_serializable(__sg_storage_file);

exit_shogun();
return 0;
}
