require 'modshogun'

f_feats_train = Modshogun::CSVFile.new "../../data/regression_1d_sinc_features_train.dat"
f_feats_inducing = Modshogun::CSVFile.new "../../data/regression_1d_sinc_features_train.dat"
f_feats_test = Modshogun::CSVFile.new "../../data/regression_1d_sinc_features_test.dat"
f_labels_train = Modshogun::CSVFile.new "../../data/regression_1d_sinc_labels_train.dat"
f_labels_test = Modshogun::CSVFile.new "../../data/regression_1d_sinc_labels_test.dat"

#![create_features]
features_train = Modshogun::RealFeatures.new f_feats_train
features_test = Modshogun::RealFeatures.new f_feats_test
labels_train = Modshogun::RegressionLabels.new f_labels_train
labels_test = Modshogun::RegressionLabels.new f_labels_test
#![create_features]

#![create_kernel_and_mean_function]
width = 1.0
kernel = Modshogun::GaussianKernel.new features_train, features_train, width
mean_function = Modshogun::ZeroMean.new 
#![create_kernel_and_mean_function]

#![create_inference]
inducing_points = NArray.sint(3)
inducing_points[0] = 0
inducing_points[1] = 1
inducing_points[2] = 2
inducing_features = Modshogun::RealFeatures.new f_feats_inducing
inducing_features.add_subset inducing_points

gauss_likelihood = Modshogun::GaussianLikelihood.new 
inference_method = Modshogun::FITCInferenceMethod.new kernel, features_train, mean_function, labels_train, gauss_likelihood, inducing_features
#![create_inference]

#![create_instance]
gp_regression = Modshogun::GaussianProcessRegression.new inference_method
#![create_instance]

#![train_and_apply]
gp_regression.train 
labels_predict = gp_regression.apply_regression features_test
#![train_and_apply]

#![compute_variance]
variances = gp_regression.get_variance_vector features_test
#![compute_variance]

#![evaluate_error]
mserror = Modshogun::MeanSquaredError.new 
mse = mserror.evaluate labels_predict, labels_test
#![evaluate_error]

# integration testing variables
output = labels_test.get_labels 

# Serialize output for integration testing (automatically generated)
__sg_storage = Modshogun::WrappedObjectArray.new 
__sg_storage_file = Modshogun::SerializableAsciiFile.new "sparse_gaussian_process_regression.dat", 'w'
__sg_storage.append_wrapped_real width, "width"
__sg_storage.append_wrapped_real_vector variances, "variances"
__sg_storage.append_wrapped_real mse, "mse"
__sg_storage.append_wrapped_real_vector output, "output"
__sg_storage.save_serializable __sg_storage_file
